require 'csv'

require_relative 'player'
require_relative 'die'
require_relative 'game_turn'
require_relative 'treasure_trove'


module StudioGame
	class Game 
		attr_reader :title 

		def initialize(title) 
			@title = title
			@players = []
		end

		def add_player(player)
			@players << player
		end

		def load_players(from_file)
			CSV.foreach(from_file) do |row|
				add_player(Player.new(row[0], row[1].to_i))
			end
		end

		def high_score_entry(player)
			formatted_name = player.name.ljust(20, '.')
			"#{formatted_name} #{player.score}"
		end

		def save_high_scores(to_file="high_scores.txt")
			File.open(to_file, "w") do |file|
				file.puts "#{@title} High Scores:\n"
				@players.sort.each do |player|
					file.puts high_score_entry(player)				
				end
			end
		end

		def play(rounds)
			puts "There are #{@players.length} in #{title}: "
			
			@players.each do |player|
				puts player
			end

			treasures = TreasureTrove::TREASURES

			puts "\nThere are #{treasures.size} treasures to be found:"
			treasures.each do |treasure| 
				puts "A #{treasure.name} is worth #{treasure.points}"
			end

			1.upto(rounds) do |round|
				if block_given? 
				    break if yield 
				end
				puts "\nRound: #{round}: "
				@players.each do |player|
					GameTurn.take_turn(player)
					puts player
				end
			end
		end

		def print_name_and_health(player)
			puts "#{player.name} (#{player.health})"
		end

		def print_stats
			strong, wimpy = @players.partition { |p| p.strong? }

			puts "#{total_points} total points for treasures found."

			@players.sort.each do |player|
				puts "\n#{player.name}'s point totals:"
				player.each_found_treasure do |treasure|
					puts "#{treasure.points} total #{treasure.name} points!"
				end
				puts "#{player.points} grand total points."
			end


			puts "\n#{@title} High Scores:"
			@players.sort.each do |player|
				puts high_score_entry(player)
			end

			puts "\n#{@title} Statistics:"

			puts "\n#{strong.length} strong players:"
			strong.each do |player|
				print_name_and_health(player)
			end

			puts "\n#{wimpy.length} wimpy players:"
			wimpy.each do |player|
				print_name_and_health(player)
			end
		end

		def total_points 
			@players.reduce(0) { |sum, player| sum += player.points }
		end
	end
end